#!/bin/bash

MODULE_FILE="$(dirname $0)/../modules.mk"
DEFAULT_BRANCH="devel"
TMP_MOD="modules.tmp"

function exec_cmd() {
	echo "Running $@"
	if ! $@ > /dev/null 2>&1; then
		echo "Failed running $@"
		exit 1;
	fi
}

function clone() {
	local url name version

	url="${1}"
	name="${2}"
	version="${3}"

	if [ -n "${CI_JOB_TOKEN}" ]; then
		exec_cmd git clone https://gitlab-ci-token:${CI_JOB_TOKEN}@$url $name
	else
		exec_cmd git clone git@${url/\//:} $name
	fi
	exec_cmd git -C $name checkout $version
}

function pull_module() {
	local line url name version tmp

	url="${2}"
	url="$(echo ${url/https:\/\//})"
	tmp="$(basename $url)"
	name="${1}/${tmp/.git/}"
	version="${3}"

	clone "$url" "$name" "$version"
}

function update_module() {
	local url name version

	version=""
	name="${1}"
	url="${2}"
	url="${url/https:\/\//}"
	url="${url/http:\/\//}"
	url="git@${url/\//:}"

	version="$(git ls-remote --heads $url ${DEFAULT_BRANCH})"
	version="$(echo $version|awk '{printf $1}')"
	if [ -z "${version}" ]; then
		echo "Failed to get SHA1 for $url"
		exit 7
	fi

	echo "$name ${2} $version" >>${TMP_MOD}
}

function pull_modules() {
	local line

	if [ -z "${CI_JOB_TOKEN}" ]; then
		echo "Using ssh credentials for pulling the plugins"
	fi

	while read -r line
	do
		pull_module $line
	done < "${MODULE_FILE}"
}

function fix_modules() {
	local line url

	if [ -f "${TMP_MOD}" ]; then
		exec_cmd rm -f ${TMP_MOD}
	fi

	while read -r line
	do
		update_module $line
	done < "${MODULE_FILE}"
	exec_cmd mv ${TMP_MOD} ${MODULE_FILE}
}

function print_help() {
	echo "$0 script to manage sulu modules for CI pipeline"
	echo "Usage"
	echo "  @0 <cmd>"
	echo ""
	echo "Options:"
	echo "  pull	Pull sulu modules in current directory defined in ${MODULE_FILE}"
	echo "  bump	Update sulu modules version in ${MODULE_FILE}"
	echo "  tar	Create a tar release for sulu"
}

function tar_release() {
	COMMITHASH=$(git rev-parse HEAD)
	FILENAME="sulu-builder-$COMMITHASH.tar.gz"
	PKG_DIRECTORY="sulu-builder-$COMMITHASH"

	[ -d "${PKG_DIRECTORY}" ] && rm -rf ${PKG_DIRECTORY}
	# Include pre-built sulu assets
	mkdir -p ${PKG_DIRECTORY}/dist/presets
	mkdir -p ${PKG_DIRECTORY}/dist/settings
	cp -R ./build/dist/* ./${PKG_DIRECTORY}/dist/
	cp ./build/device/presets/*.json ./${PKG_DIRECTORY}/dist/presets/
	cp ./build/device/settings/*.json ./${PKG_DIRECTORY}/dist/settings/

	# Include builder as well
	exec_cmd rsync -av --include=src/libs/build --exclude={'.git/','build/','node_modules/','plugins/','core/',"${PKG_DIRECTORY}/"} ./ ${PKG_DIRECTORY}/

	exec_cmd tar -zcvf $FILENAME ${PKG_DIRECTORY}
}

if [ "$1" = "pull" ]; then
	pull_modules
elif [ "$1" = "bump" ]; then
	fix_modules
elif [ "$1" = "tar" ]; then
	tar_release
else
	print_help
	exit 3
fi
